import json, requests
from urllib.request import urlopen
from urllib.parse import urlencode, unquote, quote_plus
import matplotlib.pyplot as plt
import pandas as pd
from PIL import Image
from io import BytesIO
import numpy as np

# 날씨 이미지를 now_weather.png 파일로 저장시켜주는 코드 / 매개값 : x / 리턴값 : x -> 이미지파일로 저장(출력)
def imgFile():
    url = 'http://api.openweathermap.org/data/2.5/weather'
    params = '?' + urlencode({
        quote_plus("lat"): "34.568",                                # 도시
        quote_plus("lon"): "126.977",
        quote_plus("cnt"): '5',
        quote_plus("appid"): '98399fd9226e82728bde87664fa16b59',  # 인증키
        quote_plus("mode"): 'json',                               # 반환타입
        quote_plus("units"): 'metric'                             # 섭씨온도로 자동 변환
    })

    url = url+unquote(params)
    req = requests.get(url).content                  # url로 데이터 추출함 - 바이트타입 추출
    data1 = json.loads(req)
    weatherinfo = data1['weather'][0]

    #     현재 기상정보 사진 가져와주는 코드 이후에 개발완료
    imgurl = 'http://openweathermap.org/img/wn/' + weatherinfo['icon'] + '@2x.png'
    req = requests.get(imgurl).content
    img = Image.open(BytesIO(req))
    img_matrix = np.array(img)
    plt.imshow(img_matrix)
    img.save('now_weather.png')



from urllib.request import urlopen
from urllib.parse import urlencode, quote_plus, unquote
from bs4 import BeautifulSoup
from selenium import webdriver
# Gpss.py 파일 필요 -> 딕셔너리의 키는 역이름이고 값은 좌표를 가진 데이터 파일임
import xmltodict, json
import pandas as pd
import hashlib
import hmac
import base64
import requests
import time
import numpy as np
from urllib.parse import urlencode
from urllib.request import urlopen, Request
import urllib
import bs4
import Traffic.SubDIct
import Traffic.NearStation
import Traffic.SuttleDict
import Traffic.SubTime
import Traffic.ShuttleTimeGo
import Traffic.SpanTime
import Util.Eclass



start = time.time()
dictgo = Traffic.SuttleDict.timetableGo
now_location = ['37.491575', '127.030912']
# now_location = Now_LatNLon()
nearsub1 = Traffic.NearStation.nearSubStation(now_location[0], now_location[1])  # 좌표(현재 위치) -> 정왕역 (좌표로 부터 가까운 역 검색)
index1 = nearsub1.rstrip('역')  # 정왕역 -> 정왕
# n = time.localtime().tm_wday
n = 1
weeks = ['월', '화', '수', '목', '금']
# print(weeks[n])
# data = int(class("2015162013", 'mouse3178!')[weeks[n]])
data = Util.Eclass.eclass("2019152025", 'su9191su?k')[1][weeks[n]]
h1 = int(data['hour'])
m1 = int(data['minute'])
std = Traffic.ShuttleTimeGo.shuttleTimeGo(h1, m1, dictgo)
h2 = int(std[0])
m2 = int(std[1])
dict1 = {}
dict1['starty'], dict1['startx'] = Traffic.SubDIct.subDict[index1]  # 정왕 -> 정왕역 좌표 [37.351735, 126.742989] (역이름으로 좌표 반환)
time_info = Traffic.SpanTime.spanTime(dict1['startx'], dict1['starty'])
h3, m3 = Traffic.SubTime.subTime(h2, m2, int(time_info))
Travel_time_bus = Traffic.NearStation.nearBusStation(now_location[0], now_location[1])
h4, m4 = Traffic.SubTime.subTime(h3, m3, Travel_time_bus)
h5, m5 = Traffic.SubTime.subTime(h4, m4, 10)
print("출발 시스템"+"-"*60)
print("현재 " + str(now_location) + "의 위치에서 가장 가까운 역은 " + str(index1) + "역입니다.\n")
print("출발 추천시간 (집-> 버스정류장) : " + str(h5) + "시 " + str(m5) + "분(예상소요시간 : 10분)\n")
if Travel_time_bus > 4:
    print("버스정류장->%s역 : %s시 %s분(예상소요시간 : %s분)" % (index1, str(h4), str(m4), str(Travel_time_bus)))
print(nearsub1 + "에서 출발해야하는 시간 : " + str(h3) + "시 " + str(m3) + "분(예상소요시간 : " + time_info + "분)")
print("정왕역에서 출발하는 " + str(h2) + "시 " + str(m2) + "분 셔틀 탑승")
print("학교에 도착해야하는 시간 : %d시 %d분 (시간표 정보 읽어오기)\n" % (h1, m1))
rjfflstlrks = time.time() - start
print('응답시간 = '+str(rjfflstlrks))




import json, requests
from urllib.request import urlopen
from urllib.parse import urlencode, unquote, quote_plus
import pandas as pd
from PIL import Image
import numpy as np

# 좌표를 입력받고 정보를 다 출력해주는 함수 / 매개값 : '37.56826','126.977829' / 리턴값 : (11.13, 12.0, 10.0, nan, nan, nan, 1.49) / 다른 .py에서의 사용 x
def openWeather(lat, lon):
    url = 'http://api.openweathermap.org/data/2.5/weather'
    params = '?' + urlencode({
        quote_plus("lat"): lat,                                # 도시
        quote_plus("lon"): lon,
        quote_plus("cnt"): '10',
        quote_plus("appid"): '98399fd9226e82728bde87664fa16b59',  # 인증키
        quote_plus("mode"): 'json',                               # 반환타입
        quote_plus("units"): 'metric'                             # 섭씨온도로 자동 변환
    })

    url = url+unquote(params)
    req = requests.get(url).content                  # url로 데이터 추출함 - 바이트타입 추출
    data1 = json.loads(req)
    weatherinfo = data1['weather'][0]
    wind = data1['wind']

    # 데이터 처리
    data = pd.DataFrame([data1['main']])
    data = data.transpose()
    data.columns = ['value']
    data = data['value'].to_dict()

    return (data['temp'], data['temp_max'], data['temp_min'], np.nan, np.nan, np.nan, wind['speed'])

if __name__ == "__main__":
    output = openWeather('37.56826','126.977829')
    print(output)


import hashlib
import hmac
import base64
import requests
import time
import numpy as np
from urllib.request import urlopen
from urllib.parse import urlencode, quote_plus, unquote
import json
from urllib.request import urlopen, Request
import urllib
import bs4

def ipCall():
    enc_location = urllib.parse.quote("ip 주소 확인")
    url = 'https://search.naver.com/search.naver?ie=utf8&query=' + enc_location

    req = Request(url)
    page = urlopen(req)
    html = page.read()
    soup = bs4.BeautifulSoup(html,'html5lib')
    ip=soup.select("div[class=ip_chk_box]")
    name = str(ip[0])
    answer = name[24:]
    array = answer.split("<")
    return array[0]

# Signature Make
def makeSignature(method, basestring, timestamp, access_key, secret_key):
    message = method + " " + basestring + "\n" + timestamp + "\n" + access_key
    message = bytes(message, 'utf-8')
    signature = base64.b64encode(hmac.new(secret_key, message, digestmod=hashlib.sha256).digest())

    return signature

def requestApi(timestamp, access_key, signature, uri):
    # Header for Request
    headers = {'x-ncp-apigw-timestamp': timestamp,
               'x-ncp-iam-access-key': access_key,
               'x-ncp-apigw-signature-v2': signature}

    # Geolocation API Request
    res = requests.get(uri, headers=headers)

    # Check Response
    content = res.content.decode(encoding='utf-8')
    content1 = content.split(",")
    content1=np.array(content1)
    content1[7]=content1[7].replace('"lat": ','')
    content1[8]=content1[8].replace('"long": ','')
    return content1[7],content1[8]

def reverseLocation(lat,lon):     #파라미터로 위도 경도 와야함(위도 경도로 위치정보 구하는 api)
    lat = lat  # 한국산업기술대학교 위도("37.340037")
    lon = lon  # 한국산업기술대학교 경도("126.733381")
    key = "l7xx00cd3e8e95cc4fee84f48861b2d6d812"  # api 키

    url = "https://apis.openapi.sk.com/tmap/geo/reversegeocoding?version=1"
    queryParams = "&" + urlencode({quote_plus('lat'): lat,
                                   quote_plus('lon'): lon,
                                   quote_plus('coordType'): "WGS84GEO",
                                   quote_plus('addressType'): "A01",
                                   quote_plus('appKey'): key})

    request = urllib.request.Request(url + unquote(queryParams))
    request.get_method = lambda: 'GET'
    response_body = urlopen(request).read()
    decode = response_body.decode('utf-8')
    json_data = json.loads(decode)
    return(json_data["addressInfo"]["fullAddress"])  # json을 조작하여 위도경도 다시 반환가능

def location():
    # Signature 생성에 필요한 항목
    method = "GET"
    ipname = ipCall()
    basestring = "/geolocation/v2/geoLocation?ip="+ipname+"&ext=t&responseFormatType=json"
    timestamp = str(int(time.time() * 1000))
    access_key = "1bl8GtXyTTzXDCG4lgrA"  # access key id (from portal or sub account)
    secret_key = "C43S00RfBboTCIm89TlO8DxqLzChvuLBgr0fO2SD"  # secret key (from portal or sub account)
    secret_key = bytes(secret_key, 'UTF-8')
    signature = makeSignature(method, basestring, timestamp, access_key, secret_key)

    # GET Request
    hostname = "https://geolocation.apigw.ntruss.com"
    requestUri = hostname + basestring
    (lat, lon) = requestApi(timestamp, access_key, signature, requestUri)

    return (lat, lon)

if __name__ == "__main__":
    print(location())
    print(reverseLocation(location()[0],location()[1]))


import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import requests, json
from urllib.request import urlopen
from urllib.parse import urlencode, unquote, quote_plus
# 아래 3개의 함수는 KorWeather1에서 가져와서 썼음
from Weather.KorWeather import timeInfo, mapToGrid, getData


# 강수확률이랑 시간별 강수확률을 반환해주는 함수 / 매개값 : data(json), 시간표상 하교시간 or 집 도착시간 둘 중에 결정
# 리턴값 : 등교~하교 사이의 강수확률과 강수량을 평균값으로 반환, 각 시간별 강수확률 반환 -> 22일 저녁에 강수량도 같이 넣을 예정!!!!!!!!!!!!!!!
def rainPersentage(data, end_time):
    pd.set_option('mode.chained_assignment', None)  # SettingWithCopyWarning 경고 끄는 코드
    #     print(data)
    df = data[['category', 'fcstTime', 'fcstValue']]
    #          6시간 강수량, 온도, 강수확률
    save_category = ['R06', 'T3H', 'POP']
    save_index = [i for i, ch in enumerate(df['category']) for j in save_category
                  if ch == j]
    data = df.iloc[save_index]
    data.index = range(len(data))  # 인덱스 재정의

    for i in range(len(data)):
        data.iloc[i, 1] = data.iloc[i, 1][0:2]  # 1800 -> 18로 뒤에 00때는 코드
    #     print(data2.iloc[i, 1])

    # print(dict1, '\n', data2, '\n')
    POP = data.loc[data.category == 'POP']  # 강수확률
    R06 = data.loc[data.category == 'R06']  # 6시간 강수량
    T3H = data.loc[data.category == 'T3H']  # 3시간 기온

    #     print("강수확률")
    #     print(POP)

    try:
        POP['fcstTime'] = POP['fcstTime'].astype('int')
    except:
        pass

    POP.index = np.arange(len(POP))
    R06.index = np.arange(len(R06))
    T3H.index = np.arange(len(T3H))
    strat_time = 10  # 등교시간
    end_time = 22

    sum_per = 0.0
    sum_amount = R06['fcstValue'][0]
    index = 0.0
    dict1 = {}
    for i in range(len(POP)):
        if POP['fcstTime'][i] == 0:
            POP['fcstTime'][i] = 24
        if POP['fcstTime'][i] >= end_time:
            break
                # print(POP['fcstTime'][i], POP['fcstValue'][i])
        dict1[POP['fcstTime'][i]] = POP['fcstValue'][i]
        sum_per += float(POP['fcstValue'][i])  # 비 올 확률
        index = i + 1
    try:
        return (sum_per / index, sum_amount, dict1)
    except ZeroDivisionError:
        return ("ZeroDivisionError : POP['fcstTime'][i] >= end_time를 만족하는 인덱스값이 없습니다.", "", dict1)


# rainpersentage로부터 데이터를 받아와서 그래프를 그리기위한 함수
def rainData():
    url = 'http://apis.data.go.kr/1360000/VilageFcstInfoService/getVilageFcst'
    data1 = getData(url, '37.518393', '126.737572')  # 집 위치 37.518393', '126.737572
    data2 = getData(url, '37.339873', '126.733557')  # 산기대 위치 37.339873, 126.733557
    rain_home = rainPersentage(data1, 22)  # 22 = 하교후 집 도착시간
    rain_school = rainPersentage(data2, 20)  # 20 = 하교시간

    if str(type(rain_home[0])) == "<class 'float'>":
        print("(집 기준)등교~하교시 비올 확률 : %0.1f, 예상 강수량 : %s" % (rain_home[0], rain_home[1]))
        print(rain_home[2], '\n')
        print("(학교 기준)등교~하교시 비올 확률 : %0.1f, 예상 강수량 : %s" % (rain_school[0], rain_school[1]))
        print(rain_school[2], '\n')

        # 그래프를 출력하는 코드 rain_home[2] == 시간별 강수 데이터(딕셔너리형)
        plt.plot(list(rain_home[2].keys()), list(rain_home[2].values()), 'go-', label="Home")
        plt.plot(list(rain_school[2].keys()), list(rain_school[2].values()), 'rs-', marker='*', label="KPU")
        plt.xlabel("time")
        plt.ylabel("value")
        plt.title("weather_info")
        plt.legend(loc=1)
        plt.axis([8, 22, 0, 3])
        plt.gca().invert_yaxis()
        plt.grid(True)
        plt.show()
        plt.savefig("1")

if __name__ == "__main__":
    rainData()


import pandas as pd

pd.options.display.float_format = '{:.1f}'.format  # 소수점 1자리만 나타내주는 코드
dt = pd.read_excel("test.xlsx")
dt['now_time'] = dt['now_time'].fillna(method='bfill')
dt['time'] = dt['time'].fillna(method='bfill')
print(dt)
print(dt.loc[dt['type']=='통합'][53::])


from urllib.request import urlopen
from urllib.parse import quote_plus, unquote
import xmltodict, json,  Traffic.SubDIct
import pandas as pd
import requests
from urllib.parse import urlencode
from urllib.request import urlopen
import urllib

def nearSubStation(lat, lon):
    lat = lat  # 한국산업기술대학교 위도("37.340037")
    lon = lon  # 한국산업기술대학교 경도("126.733381")
    key = "l7xx00cd3e8e95cc4fee84f48861b2d6d812"  # api 키
    url = "https://apis.openapi.sk.com/tmap/pois/search/around?version=1"
    queryParams = "&" + urlencode({quote_plus('centerLat'): lat,
                                   quote_plus('centerLon'): lon,
                                   quote_plus('radius'): "0",
                                   quote_plus('categories'): '%EC%A7%80%ED%95%98%EC%B2%A0%EC%97%AD',
                                   quote_plus('appKey'): key})

    request = urllib.request.Request(url + unquote(queryParams))
    request.get_method = lambda: 'GET'

    response_body = urlopen(request).read()
    decode = response_body.decode('utf-8')

    try:
        json_data = json.loads(decode)
        data = json_data['searchPoiInfo']['pois']['poi'][0]['name']
        stloc = data.rfind('역') + 1
        return data[:stloc]
    except:
        return "err"


def nearBusStation(lat, lon):
    url = 'http://ws.bus.go.kr/api/rest/pathinfo/getPathInfoByBus'
    params = '?' + urlencode({
        quote_plus(
            "serviceKey"): "p%2FjvMYI4C7znYs1z7bwkpbC5XOPhQ2c5XMyRYcH6A%2BLnvRkIO95%2F9tyKecxg06ais5rGNSxsY6eplXsP3%2F%2FiSw%3D%3D",
        quote_plus("startX"): lon,  # 현재위치가 들어갈 곳
        quote_plus("startY"): lat,  # 현재위치가 들어갈 곳
        quote_plus("endX"): Traffic.SubDIct.subDict[nearSubStation(lat, lon)[:-1]][1],  # 이 좌표는 현재위치로부터 가까운 역인 강남역의 좌표임
        quote_plus("endY"): Traffic.SubDIct.subDict[nearSubStation(lat, lon)[:-1]][0],
    })

    url = url + unquote(params)
    req = requests.get(url)
    xpars = xmltodict.parse(req.text)
    dump = json.dumps(xpars)
    data1 = json.loads(dump)

    Travel_time = data1['ServiceResult']['msgBody']['itemList']
    data = pd.DataFrame(Travel_time)
    data['time'] = data['time'].astype('int')
    Travel_time_bus = int(data['time'].mean(axis=0) + 0.5)  # 평균 소요시간
    return Travel_time_bus


import Weather.NaverWeather as na
import Weather.KorWeather as ka
import Weather.OpenWeather as oa
import pandas as pd
import numpy as np
import datetime, pytz, schedule, time

def weatherZip():
    pd.set_option('mode.chained_assignment', None)  # SettingWithCopyWarning 경고 끄는 코드
    home_lat = '37.518393'
    home_lon = '126.737572'
    lat = "37.3398"
    lon = "126.7335"
    nadata = na.naverWeather(lat, lon)
    kodata = ka.korWeather(home_lat, home_lon, lat, lon)
    ondata = oa.openWeather(lat, lon)
    now_time=str()

    try:
        kodata_backup = kodata[-2:]
        kodata = kodata[:-2]
        now_time = datetime.datetime.now(tz=pytz.timezone('Asia/Seoul')).strftime('%m월%d일 %H시%M분%S초')
        data = pd.DataFrame([nadata, kodata, ondata], columns=['현재온도', '최고온도', '최저온도', '비올확률', '강수량', '습도', '풍속'], index=['네이버 날씨', '기상청 날씨', '오픈 웨더 날씨'])
        weight = [15, 3, 2]  # 가중치 설정

        # 가중치 만큼 복사시키는 코드
        for i in np.arange(weight[0]):
            data.loc[i] = data.loc['네이버 날씨']  # data.index[0]

        for i in (np.arange(weight[1]) + weight[0]):
            data.loc[i] = data.loc['기상청 날씨']  # data.index[1]

        for i in (np.arange(weight[2]) + weight[0] + weight[1]):
            data.loc[i] = data.loc['오픈 웨더 날씨']  # data.index[2]

        print(data.head(3), "\n");

        data = data.astype('float')
        info = pd.DataFrame(data.mean(axis = 0), columns = ['가중치 적용한 평균값'])
        pd.options.display.float_format = '{:.1f}'.format  # 소수점 1자리만 나타내주는 코드
        info = info.transpose()
        info['time'] = [ka.timeInfo()]
        info['now_time'] = now_time
        # print(kodata)
        # print(kodata_backup)
        info['home_rain_per'] = kodata_backup[-2][0]
        info['home_rain_amount'] = kodata_backup[-2][1]
        info['school_rain_per'] = kodata_backup[-1][0]
        info['school_rain_amount'] = kodata_backup[-1][1]
        # pd.set_option('display.max_columns', 10)   # n개의 컬럼을 다 보여줘라~
        print(info[['현재온도', '비올확률', '강수량', 'now_time']])

        # 최초 실행시
        # dt = info.to_excel("test.xlsx", encoding='euc-kr')
        # 2번째 실행시
        dt = pd.read_excel("test.xlsx")
        dt = pd.concat([dt, data.head(3)], ignore_index=True)
        dt = pd.concat([dt, info], ignore_index=True)
        dt.to_excel("test.xlsx", encoding='euc-kr', index=False)
        print("test.xlsx에 저장 완료")
        print("------------------------------------------------------------------------")
        return

    except IOError:
        f = open("test_error.txt", "a")
        print("kodata : ", kodata)
        print("IOError에러발생 : test.xlsx파일이 열려있다면 닫아주세요\n" + now_time)
        f.write("IOError에러발생 : test.xlsx파일이 열려있다면 닫아주세요\n" + now_time)
        return


    except ValueError:
        print("kodata : ", kodata)
        f = open("test_error.txt", "a")
        print("ValueError에러발생 : " + now_time + '\n' + str(ka.timeInfo()) + '\n' + "ka.main2(home_lat, home_lon, lat, lon) 리턴값 : 오류가났습니다", kodata, '\n')
        f.write("ValueError에러발생 : " + now_time + '\n' + str(ka.timeInfo()) + '\n' + "ka.main2(home_lat, home_lon, lat, lon) 리턴값 : 오류가났습니다\n", kodata, '\n')
        return

    except ValueError:
        print("kodata : ", kodata)
        f = open("test_error.txt", "a")
        print("ValueError : could not convert string to float: '오'" + now_time + '\n' + str("ValueError : could not convert string to float: '오'" + '\n' + "ka.main2(home_lat, home_lon, lat, lon) 리턴값 : 오류가났습니다", kodata, '\n'))
        f.write("ValueError에러발생 : " + now_time + '\n' + str(ka.time_info()) + '\n' + "ka.main2(home_lat, home_lon, lat, lon) ValueError : could not convert string to float: '오'\n", kodata, '\n')
        return

        print()
    except:  # 예외 발생 확인을 위해 주석처리해둠
        print("kodata : ", kodata)
        f = open("test_error.txt", "a")
        print("에러발생 : " + now_time +'\n' + str(ka.timeInfo()) + '\n'  + kodata + '\n')
        f.write("에러발생 : " + now_time +'\n' + str(ka.timeInfo()) + '\n' + kodata + '\n')
        return

if __name__ == "__main__":
    weatherZip()
    schedule.every(1).minutes.do(weatherZip)
    # # schedule.every(10).seconds.do(weatherZip)
    while True:
        schedule.run_pending()
    #     # time.sleep(10)



from urllib.request import urlopen, Request
from bs4 import BeautifulSoup as bs
from urllib.parse import quote, urlencode, quote_plus, unquote
import urllib, re, json
from Util.Gpss import reverseLocation

# 좌표값으로 웹크롤링한 데이터를 반환해준다. / 매개값 : ("37.3398", "126.7335") / 리턴값 : 좌표와 날씨의 키워드로 검색한 모든 데이터 / 다른 .py파일에서의 사용 x
def getData(lat, lon):
    location = reverseLocation(lat, lon)  # resurn : 경기도 시흥시 정왕1동
    url = 'https://search.naver.com/search.naver?ie=utf8&query=' + quote(location + '날씨')
    # print(url)
    req = Request(url)
    page = urlopen(req)
    html = page.read()
    soup_text = bs(html, 'html5lib')
    # print(soup_text)
    return soup_text

# 2. 기상정보 출력, 3. 강수유무 출력 / 매개값 : ("37.3398", "126.7335") / 리턴값 : ('1', '6', '2', '30', 0.0, '75', '1') / 다른 .py파일에서의 사용 x
def naverWeather(lat, lon):
    s = getData(lat, lon)

    # 맑음, 어제보다 1도 낮아요 - 모니터 인터페이스로 구성
    t2 = s.find('ul', class_='info_list').find('p', class_='cast_txt').text
    # 비올 확률 (미래)  .find_all('dd', class_='weather_item _dotWrapper') - 개발중
    # rain_hour = str(s.find_all('li', class_= 'on merge1'))
    # print(type(rain_hour))

    # 현재 기온
    tnow = s.find('p', class_='info_temperature').find('span', class_='todaytemp').text  # + '℃'
    # 최고 기온
    tmax = s.find('span', class_='merge').find('span', class_='max').find('span', class_='num').text  # + '℃'
    # 최저 기온
    tmin = s.find('span', class_='merge').find('span', class_='min').find('span', class_='num').text  # + '℃'
    # 현재 비올 확률
    rain = s.find('li', class_='on now merge1').find('dd', class_='weather_item _dotWrapper').text
    # 현재 강수량
    rain_data = s.find('li', class_='on now merge1').find('dd', class_='item_condition').text.strip()
    rain_amount = (float(rain_data[0])+float(rain_data[-1]))/2
    # 현재 습도
    reh = s.find('div', class_='info_list humidity _tabContent _center').find('dd', class_='weather_item _dotWrapper').find('span').text
    # 현재 풍속
    wind = s.find('div', class_='info_list wind _tabContent _center').find('dd', class_='weather_item _dotWrapper').find('span').text  # + 'm/s'

    #   현재온도, 최고온도, 최저온도, 비올확률,       강수량,   습도, 풍속
    return (tnow, tmax, tmin, rain.strip().replace('%', ''), rain_amount, reh, wind)

# 경지가 개발한 좌표를 주소로 바꿔주는 코드 / 매개값 : "37.340037", "126.733381" / 리턴값 : 경기도 시흥시 정왕1동 / 아마 다른 파일에서도 사용할 듯

if __name__ == "__main__":
    print(naverWeather("37.3398", "126.7335"))  # 학교 온도 개발 완료


from urllib.parse import urlencode, unquote, quote_plus
import Util.Gpss
import json, math, datetime, pytz, requests
import pandas as pd

# 현재 시간에서 1시간씩 빼면서 기상청이 데이터를 갱신해주는 시간을 찾아서 반환해주는 코드 / 다른 .py파일에서의 사용 x / 매개값 : 없음 / 리턴값 : str타입 (20201102, 2200)
def timeInfo():
    standard_time = [2, 5, 8, 11, 14, 17, 20, 23] # 기상청이 데이터 갱신해주는 시간
    time_now = datetime.datetime.now(tz=pytz.timezone('Asia/Seoul')).strftime('%H') #현재 서울의 시간 ex)11, 22
    check_time = int(time_now) - 1 #현재 시간에서 1시간 뺀 값 ex) 21 - 1 = 20시 이유 : 데이터가 40분에 업데이트 돼서 ex) 21시 40분

    day_calibrate = 0
    while not check_time in standard_time : # 기상청이 데이터 갱신해주는 시간과 같을때까지 현재시간 =- 1
        check_time -= 1
        if check_time < 2 : # 만약 2시보다 작다면 그 전인 23시의 데이터를 받아와야 하므로
            day_calibrate = 1  # 이후 날짜의 계산에서 하루전으로 만들어주려고 나중에 1을 빼면 되므로 1을 저장
            check_time = 23    # 2시 이전의 standard_time은 23시 이므로 그냥 값을 줘버림

    date_now = datetime.datetime.now(tz=pytz.timezone('Asia/Seoul')).strftime('%Y%m%d')  #현재 시간의 년도, 월, 일 ex) 20200101
    check_date = int(date_now) - day_calibrate #만약 02시 이전이여서 day_calibrate이 값이 1로 됐으면 날짜를 하루 빼줘야하므로
    if check_time < 10:
        return (str(check_date), ('0' + str(check_time) + '00'))  # ex) (20200109, 시각00) 00을 붙이는 이유는 api에서 데이터의 값으로 2200을 요구해서이다.
    else:
        return (str(check_date), (str(check_time) + '00'))  # ex) (20200109, 시각00) 00을 붙이는 이유는 api에서 데이터의 값으로 2200을 요구해서이다.

# 강수형태 -> 차후 겨울에 눈올때 서비스 예정이긴 한데 이미지 파일로 대체도 가능 / 다른 .py파일에서의 사용 x / PTY는 강수타입을 말하는 거임 강수타입인 key값에 따라 value값을 반환 현재 사용X 차후 필요시 개발
def rainType(PTY):
    dict = {'0': '없음', '1': '비', '2': '비/눈', '3': '눈', '4': '소나기', '5': '빗방울', '6': '빗방울/눈날림', '7': '눈날림'}
    return dict[PTY]

# 일반적인 좌표값을 기상청의 격자 좌표로 변경해주는 코드 / 다른 .py파일에서의 사용 x / 매개값 : float타입 (37.339873, 136.733557) / 반환값 : int타입 (228, 134)
def mapToGrid(lat, lon, code=0):
    NX = 149  ## X축 격자점 수
    NY = 253  ## Y축 격자점 수

    Re = 6371.00877  ##  지도반경
    grid = 5.0  ##  격자간격 (km)
    slat1 = 30.0  ##  표준위도 1
    slat2 = 60.0  ##  표준위도 2
    olon = 126.0  ##  기준점 경도
    olat = 38.0  ##  기준점 위도
    xo = 210 / grid  ##  기준점 X좌표
    yo = 675 / grid  ##  기준점 Y좌표
    PI = math.asin(1.0) * 2.0
    DEGRAD = PI / 180.0
    RADDEG = 180.0 / PI

    re = Re / grid
    slat1 = slat1 * DEGRAD
    slat2 = slat2 * DEGRAD
    olon = olon * DEGRAD
    olat = olat * DEGRAD

    sn = math.tan(PI * 0.25 + slat2 * 0.5) / math.tan(PI * 0.25 + slat1 * 0.5)
    sn = math.log(math.cos(slat1) / math.cos(slat2)) / math.log(sn)
    sf = math.tan(PI * 0.25 + slat1 * 0.5)
    sf = math.pow(sf, sn) * math.cos(slat1) / sn
    ro = math.tan(PI * 0.25 + olat * 0.5)
    ro = re * sf / math.pow(ro, sn)

    ra = math.tan(PI * 0.25 + lat * DEGRAD * 0.5)
    ra = re * sf / pow(ra, sn)
    theta = lon * DEGRAD - olon
    if theta > PI:
        theta -= 2.0 * PI
    if theta < -PI:
        theta += 2.0 * PI
    theta *= sn
    x = (ra * math.sin(theta)) + xo
    y = (ro - ra * math.cos(theta)) + yo
    x = int(x + 1.5)
    y = int(y + 1.5)
    # print(lat, lon, x, y)
    return x, y

# 좌표를 입력받아 getdata에 넘겨줘서 데이터를 수집하고 이 메소드에선 데이터 처리만함 / 다른 .py파일에서의 사용 x
# / 매개값 : float타입 ('37.518393', '126.737572', '37.339873', '126.733557', )
# / 반환값 : str타입 ('4', 9.5, -4.0, 1.3333333333333333, '0', '81', '4', (14.0, '0'), (12.666666666666666, '0'))
def korWeather(school_location1, school_location2, home_location1, home_location2):

    # Setting for URL parsing
    url1 = 'http://apis.data.go.kr/1360000/VilageFcstInfoService/getUltraSrtNcst'
    data1 = getData(url1, home_location1, home_location2)
    url2 = 'http://apis.data.go.kr/1360000/VilageFcstInfoService/getVilageFcst'
    data2 = getData(url2, home_location1, home_location2)     # 집의 날씨 예보
    data3 = getData(url2, school_location1, school_location2) # 학교의 날씨 예보

    dict1 = {}
    dict2 = {}
    if str(type(data2)) == "<class 'str'>" or str(type(data1)) == "<class 'str'>":
        # print("오류가났습니다")
        return "오류가났습니다"

    else:
        df_sample = data1[['category', 'obsrValue']]
        df_sample.index = df_sample['category']
        data11 = df_sample['obsrValue']
        dict1 = data11.to_dict()


        df = data2[['category', 'fcstTime', 'fcstValue']]
        #  6시간 강수량     온도      습도      풍속     강수확률      6시간 강수량     최고기온   최저기온
        save_category = ['R06', 'T3H', 'REH', 'WSD', 'POP', 'R06', 'TMX', 'TMN']
        save_index = [i for i, ch in enumerate(df['category']) for j in save_category if ch == j]
        data22 = df.iloc[save_index]
        data22.index = range(len(data22))  # 인덱스 재정의

        for ch in save_category:
            tmp = data22.loc[data22.category == ch].fcstValue
            tmp.index = range(len(tmp))
            tmp = tmp.to_dict()
            dict2[ch] = tmp
            # print(ch, dict2[ch])
            sum = 0.0
            for i in range(len(dict2[ch])):
                sum += float(dict2[ch][i])
                if i>=5: break
            dict2[ch] = sum/len(dict2[ch])
        # print(dict1, '\n', dict2, '\n')

        return (dict1['T1H'], dict2['TMX'], dict2['TMN'], dict2['POP'], dict1['RN1'], dict1['REH'], dict1['WSD'])

# url과 좌표를 이용해서 data를 가져오는 코드  /  다른 .py파일에서의 사용 x /  이하 매개값 str 타입
# http://apis.data.go.kr/1360000/VilageFcstInfoService/getUltraSrtNcst 37.339873 126.733557
# http://apis.data.go.kr/1360000/VilageFcstInfoService/getVilageFcst 37.339873 126.733557
# http://apis.data.go.kr/1360000/VilageFcstInfoService/getVilageFcst 37.518393 126.737572
# 반환값 DataFrame타입 :    baseDate baseTime category  nx   ny obsrValue
# 0  20201120     0800      PTY  56  122         0
# 1  20201120     0800      REH  56  122        70
#      baseDate baseTime category  fcstDate fcstTime fcstValue  nx   ny
# 0    20201120     0800      POP  20201120     1200        30  56  122
# 1    20201120     0800      PTY  20201120     1200         0  56  122
def getData(url, location1, location2):
    base = timeInfo()
    narray = mapToGrid(float(location1), float(location2))
    params = '?' + urlencode({
        quote_plus("serviceKey"): "p%2FjvMYI4C7znYs1z7bwkpbC5XOPhQ2c5XMyRYcH6A%2BLnvRkIO95%2F9tyKecxg06ais5rGNSxsY6eplXsP3%2F%2FiSw%3D%3D",     # 인증키
        quote_plus("numOfRows"): "150",  # 한 페이지 결과 수 // default : 10
        quote_plus("pageNo"): "1",  # 페이지 번호 // default : 1
        quote_plus("dataType"): "JSON",  # 응답자료형식 : XML, JSON
        quote_plus("base_date"): base[0],  # 발표일자 // yyyymmdd  20201103
        quote_plus("base_time"): base[1],  # 발표시각 // HHMM, 매 시각 40분 이후 호출
        quote_plus("nx"): narray[0],  # 예보지점 X 좌표
        quote_plus("ny"): narray[1]  # 예보지점 Y 좌표
    })

    try:
        url = url + unquote(params)
        req = requests.get(url).content  # url로 데이터 추출함 - 바이트타입 추출
        data1 = json.loads(req)  # 위에서 뽑은 바이트타입 데이터를 딕셔너리로 변경해주는 코드
        # print(data1['response']['body']['items']['item'])
        data = pd.DataFrame(data1['response']['body']['items']['item'])
    except KeyError:
        data = "KeyError에러 getdata()에 base_time 파라미터값 오류" + str(base) + "\n"
    except json.decoder.JSONDecodeError:
        data = "json.decoder.JSONDecodeError 에러 발생 이유 찾는 중..." + str(base) + "\n"
    except:
        data = "KeyError에러 getdata()에 base_time 파라미터값 오류" + str(base) + "\n"

    # print(data)
    return data

if __name__ == "__main__":
    # location = Util.gpss.main()  # location = [0] : 정왕1동, [1] : 37.340288, [2] : 126.743981 튜플반환
    # print('<현재 위치의 날씨정보>\n' + korWeather(location[1], location[2])) # gps값 들어갈 곳
    # print(korWeather('37.339873', '136.733557'))  # 임시 위치 - 데이터 확인용
    print(korWeather('37.518393', '126.737572', '37.339873', '126.733557'))  # 학교 위치
    #('37.339873', '126.733557')   #학교 좌표 위경도 값 - 구글에서 따옴


import requests, datetime, pytz, json

# visual-crossing-weather api사용 비올확률 데이터가 2가지 밖에 없어서 1가지더 추가하기 위해 11월 22일 개발중 데이터는 다 뽑았고 weatherzip에서 가중치 두고 결과 봐야함
def visualCrossing(lat, lon):
    url = "https://visual-crossing-weather.p.rapidapi.com/forecast"
    time = datetime.datetime.now(tz=pytz.timezone('Asia/Seoul')).strftime('%Y%m%d')
    location = lat + "," + lon
    # location = "38.2651,126.4852"
    querystring = {"startDateTime" : time,
                    "location" : location,
                   "aggregateHours":"24",
                   "contentType":"json",
                   "shortColumnNames":"1",
                   "unitGroup":"uk"}

    headers = {
        'x-rapidapi-key': "64116e543bmshff42d2dff0a85abp1a1265jsn7ff77ead4b84",
        'x-rapidapi-host': "visual-crossing-weather.p.rapidapi.com"
    }
    print(url)
    response = requests.request("GET", url, headers=headers, params=querystring)

    text = response.text
    data = json.loads(text)
    return data['locations'][location]['values'][0]['pop']

print(visualCrossing("37.7398", "126.7335"))


from bs4 import BeautifulSoup
from selenium import webdriver
import re
import pandas as pd
import Util

def eclass(id,password):

    #크롬창을 띄우지 않는 옵션
    options = webdriver.ChromeOptions()
    options.add_argument('headless')
    options.add_argument('disable-gpu')

    # 위치 지정
    driver = webdriver.Chrome("Chromedriver(win).exe", options=options)

    # 웹 자원 로드를 위해 암묵적으로 딜레이
    delay_time = 3
    driver.implicitly_wait(delay_time)

    # URL 접속 시도
    driver.get('http://eclass.kpu.ac.kr/ilos/main/member/login_form.acl')

    # ID, PW 입력
    # usr_id = input("ID를 입력해주세요 :")
    # usr_pwd = input("PW를 입력해주세요 :")
    usr_id = id
    usr_pwd = password
    driver.find_element_by_name('usr_id').send_keys(usr_id)
    driver.find_element_by_name('usr_pwd').send_keys(usr_pwd)

    # 로그인 버튼 클릭
    driver.find_element_by_xpath('//*[@id="myform"]/div/div/div/div').click()

    # URL 접속 시도
    driver.get("http://eclass.kpu.ac.kr/ilos/main/main_form.acl")

    html = driver.page_source
    soup = BeautifulSoup(html, 'html.parser')

    # notices = driver.find_element_by_class_name('title-01').text
    # print(notices)

    # notices = soup.select('div.p_inr > div.p_info > a > span')
    name1 = soup.select('div > ol > li > em')
    time1 = soup.select('div > ol > li > span')

    eclass = []
    for item in zip(name1, time1):
        eclass.append(
            {
                'name': item[0].text.replace('\n', '').strip(),
                'time': item[1].text.replace('\n', '').strip()
            }
        )

    data = pd.DataFrame(eclass)
    index = 0
    for i, ch in enumerate(data['name']):
        data['name'][i] = data['name'][i][:-42]

    tmp_index = []
    for i, ch in enumerate(data['time']):
        if len(ch) == 0:
            tmp_index.append(i)  # sdu index를 리스트로 저장

        elif ch[0] not in ['월', '화', '수', '목', '금']:
            #         print(index)
            index = i
            break

    for i in tmp_index:
        #     print(i)
        data.drop(data.index[i], inplace=True)
        data.index = range(len(data))  # 인덱스를 새롭게 매기는? 코드
    data.drop(data.index[index - len(tmp_index)::], inplace=True)

    dict1 = {}
    weeks = ['월', '화', '수', '목', '금']
    for i, ch in enumerate(data['time']):
        for j in range(len(data['time'][i])):
            if ch[j] in weeks:
                data['time'][i] = ch.strip()
                if ch[j] not in dict1:
                    dict1[ch[j]] = list()  # ch[j]키가 없을떈 빈리스트를 value로 하고, ch[j]를 키값으로 하는 요소생성
                if len(data['time'][i]) > 31:  # 일주일에 2번 나눠서 수업
                    if j > 1:
                        dict1[data['time'][i][0:j][0]].append([data['name'][i], data['time'][i][0:j]])
                        dict1[ch[j]].append([data['name'][i], data['time'][i][j:]])
                else:  # 일주일에 1타임만 수업
                    dict1[ch[j]].append([data['name'][i], data['time'][i]])

    for ch in dict1.keys():
        for i in range(len(dict1[ch])):
            list2 = dict1[ch][i][1].split(' ')
            if len(list2) > 2:
                if list2[2] != '':
                    dict1[ch][i][1] = list2[2]
                else:
                    dict1[ch][i][1] = list2[5]
            else:
                dict1[ch][i][1] = list2[0]

        dict1[ch] = sorted(dict1[ch], key=lambda x: x[1])

    dict2 = {}
    for ch in dict1.keys():
        dict2[ch] = dict1[ch][0][1][0:5]
        dict2[ch] = {'hour': int(dict2[ch][0:2]), 'minute': int(dict2[ch][3:5])}

    return [dict1,dict2]
if __name__ == "__main__":
    print(eclass(2019152015,"hoho1023!"))



def shuttleTimeGo(hour, minute, dict_go):
    # defhour, defminute이 선언되지 않아서 if문에서 조건을 충족하지 못해서
    # defhour이 원하는 값으로 초기화되지 않은 경우 초기화된 값이 없으므로 return에서 오류(UnboundLocalError) 발생
    defhour = int(hour)
    defminute = int(minute)
    convminute = int(minute) + 60 * int(hour)
    while (True):
        if (8 <= defhour <= 22):
            if (int(dict_go[defhour][0]) + defhour * 60 + 15 > convminute):
                # 셔틀시간+15분이 학교에 도착하는 시간이라는 가정하에,시간대에 가장 이른 셔틀의 도착시간은 defminute보다 크면 안됨,
                defhour -= 1
                # 따라서 defhour을 1 빼주고
                continue
                # defminute를 60으로 설정.
                # 반복문의 맨 처음으로 돌아가게 한다.
            else:
                for i in range(len(dict_go[defhour])):
                    if (convminute >= int(dict_go[defhour][-1 - i]) + defhour * 60 + 15):
                        defminute = dict_go[defhour][-1 - i]
                        return (str(defhour), str(defminute))
                    else:
                        continue
        else:
            return ("err", "err")




import setuptools

setuptools.setup(
    name="ToBi_pkg",
    version="0.0.1",
    author="KPU_ToBi",
    author_email="golapaduck29@naver.com",
    description="A package for KPU_ToBi",
    url="https://github.com/golapaduck",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.8",
    ],
)


from urllib.parse import quote_plus, unquote
import xmltodict, json
import requests
from urllib.parse import urlencode


def spanTime(Near_Station_Lat, Near_Station_Lon):
    url = 'http://ws.bus.go.kr/api/rest/pathinfo/getPathInfoBySubway'
    params = '?' + urlencode({
        quote_plus(
            "serviceKey"): "p%2FjvMYI4C7znYs1z7bwkpbC5XOPhQ2c5XMyRYcH6A%2BLnvRkIO95%2F9tyKecxg06ais5rGNSxsY6eplXsP3%2F%2FiSw%3D%3D",
        quote_plus("startX"): Near_Station_Lat,  # 좌표126.74
        quote_plus("startY"): Near_Station_Lon,  # 37.351828
        quote_plus("endX"): '126.74',  # 정왕역 좌표127.027637
        quote_plus("endY"): '37.351828',  # 37.497950
    })
    url = url + unquote(params)
    req = requests.get(url)
    xpars = xmltodict.parse(req.text)
    dump = json.dumps(xpars)
    data1 = json.loads(dump)
    time = data1['ServiceResult']['msgBody']['itemList']['time']
    return str(time)

